#include <stdlib.h>
#include <dlfcn.h>
#include <iostream>
#include <shared.hh>

using namespace std;

int main(int argc, char* argv[])
{
    //
    if (argc != 2)
    {
        cerr << "Usage: " << argv[0] << " library_name.so\n";
        exit(EXIT_FAILURE);
    }
    //
    void* handle = dlopen(argv[1], RTLD_LAZY);
    if (handle == NULL)
    {
	char* error_msg = dlerror();
	cerr << (error_msg ? error_msg : "Can't load shared lib") << endl;
	exit(EXIT_FAILURE);
    }
    // Get pointer to 'create_shared'
    shared* (*exp_create_shared)(void) = (shared* (*)(void))(dlsym(handle, "create_shared"));
    if (exp_create_shared == NULL)
    {
	char* error_msg = dlerror();
	if (error_msg)
	{
	    cerr << error_msg << endl;
	    exit(EXIT_FAILURE);
	}
	// not an error ??
    }
    //
    cout << argv[0] << ": Ready to create 'shared' object\n";
    shared* sh_obj = (*exp_create_shared)();
    sh_obj->foo();
    delete sh_obj;
    //
    cout << argv[0] << ": after shared::foo()\n";
    dlclose(handle);
    //
    return EXIT_SUCCESS;
}
